/**
 * Test case for define.
 * Runs with mocha.
 */
'use strict'

const define = require('../lib/define.js')
const assert = require('assert')
const { moduleProxy } = require('apemanmock')
const co = require('co')

describe('define', function () {
  this.timeout(18000)

  before(() => co(function * () {

  }))

  after(() => co(function * () {

  }))

  it('Create an instance', () => co(function * () {
    let module = define({
      logging: true
    })
    let api = moduleProxy(module, { ctx: { state: {} } })
    assert.ok(api)
  }))

  it('Call API methods', () => co(function * () {
    let module = define({
      logging: true,
      publishDir: `${__dirname}/../tmp/testing-public`
    })
    let api = moduleProxy(module, { ctx: { state: {} } })

    {
      let pong = yield api.ping('The pong!')
      assert.equal(pong, 'The pong!')
    }
    {
      let srcDir = `${__dirname}/../misc/mocks`
      yield api.scss(srcDir)
    }
    {
      let srcDir = `${__dirname}/../misc/mocks`
      yield api.bundle(srcDir)
      yield api.extractMaps('mock-script-01.js', { ignore: '*.cc.js' })
      yield api.closureCompile('mock-script-01.js')
    }
    {
      let src = require.resolve('../misc/mocks/mock-manifest.js')
      yield api.manifest(src)
    }
  }))

  it('Compare spec and implement', () => co(function * () {
    let module = define({
      logging: false
    })
    let implNames = Object.keys(module).filter((name) => !/^[\$_]/.test(name))
    let specNames = Object.keys(module.$spec)
    for (let implName of implNames) {
      assert.ok(!!~specNames.indexOf(implName), `${implName} is implemented, but not described in "$spec"`)
    }
    for (let specName of specNames) {
      assert.ok(!!~implNames.indexOf(specName), `${specName} is described in "$spec", but not implemented`)
    }
  }))
})

/* global describe, before, after, it */
